"""Tests for SmartHub MCP extension."""
import os
import pytest
from unittest.mock import Mock, patch
from fastapi.testclient import TestClient

from smarthub_extension.config import Config
from smarthub_extension.server import app
from smarthub_extension.types import ConnectionResponse, TablesResponse, MerchantResponse

# Create test client
client = TestClient(app)

# Test configuration
@pytest.fixture
def mock_config():
    """Create a mock configuration for testing."""
    return Config(
        snowflake_user="test_user",
        snowflake_account="test_account",
        snowflake_role="test_role",
        snowflake_warehouse="test_warehouse",
        log_file="/tmp/test_smarthub_mcp.log",
        debug=True
    )

# Test connection response
@pytest.fixture
def mock_connection_response():
    """Create a mock successful connection response."""
    return ConnectionResponse(
        status="success",
        role="test_role",
        message=None
    )

# Test tables response
@pytest.fixture
def mock_tables_response():
    """Create a mock successful tables response."""
    return TablesResponse(
        status="success",
        tables=[{
            "name": "TEST_TABLE",
            "schema": "PUBLIC",
            "database": "APP_MERCH_GROWTH",
            "kind": "TABLE"
        }],
        message=None
    )

# Test merchant response
@pytest.fixture
def mock_merchant_response():
    """Create a mock successful merchant response."""
    return MerchantResponse(
        status="success",
        summary={
            "merchant_token": "TEST123",
            "business_id": "12345",
            "business_name": "Test Business",
            "current_am": "Test AM",
            "am_team": "Test Team",
            "is_current": True,
            "last_updated": "2025-03-25"
        },
        details={
            "ownership": {
                "merchant_token": "TEST123",
                "business_id": "12345",
                "active_owner_id": "OWNER123",
                "has_unexpired_dm": True,
                "time_start": "2025-01-01",
                "time_end": None,
                "is_current": True
            }
        },
        data_sources=["DIM_AM_OWNERSHIP_HISTORICAL"],
        message=None
    )

@pytest.mark.asyncio
async def test_test_snowflake_connection(mock_config):
    """Test the Snowflake connection test function."""
    with patch("smarthub_extension.server.get_snowflake_connection") as mock_get_conn:
        # Setup mock cursor
        mock_cursor = Mock()
        mock_cursor.fetchone.return_value = ["test_role"]
        mock_conn = Mock()
        mock_conn.cursor.return_value = mock_cursor
        mock_get_conn.return_value = mock_conn
        
        response = client.post("/mcp/test_snowflake_connection")
        assert response.status_code == 200
        result = response.json()
        
        assert result["status"] == "success"
        assert result["role"] == "test_role"

@pytest.mark.asyncio
async def test_list_available_tables(mock_config):
    """Test the list available tables function."""
    with patch("smarthub_extension.server.get_snowflake_connection") as mock_get_conn:
        # Setup mock cursor
        mock_cursor = Mock()
        mock_cursor.fetchall.return_value = [
            ["APP_MERCH_GROWTH", "TEST_TABLE", "PUBLIC", "TABLE"]
        ]
        mock_conn = Mock()
        mock_conn.cursor.return_value = mock_cursor
        mock_get_conn.return_value = mock_conn
        
        response = client.post("/mcp/list_available_tables")
        assert response.status_code == 200
        result = response.json()
        
        assert result["status"] == "success"
        assert len(result["tables"]) == 1
        assert result["tables"][0]["name"] == "TEST_TABLE"

@pytest.mark.asyncio
async def test_get_merchant_info(mock_config):
    """Test the get merchant info function."""
    with patch("smarthub_extension.server.get_snowflake_connection") as mock_get_conn:
        # Setup mock cursor
        mock_cursor = Mock()
        mock_cursor.fetchone.return_value = [
            "TEST123", "12345", "OWNER123", True, "2025-01-01", None, True
        ]
        mock_cursor.description = [
            ("merchant_token", None), ("business_id", None),
            ("active_owner_id", None), ("has_unexpired_dm", None),
            ("time_start", None), ("time_end", None), ("is_current", None)
        ]
        mock_conn = Mock()
        mock_conn.cursor.return_value = mock_cursor
        mock_get_conn.return_value = mock_conn
        
        response = client.post("/mcp/get_merchant_info/TEST123")
        assert response.status_code == 200
        result = response.json()
        
        assert result["status"] == "success"
        assert result["summary"]["merchant_token"] == "TEST123"
        assert "DIM_AM_OWNERSHIP_HISTORICAL" in result["data_sources"]


"""Common types used across the SmartHub extension."""
from typing import Dict, List, Optional, Union
from typing_extensions import TypedDict

class MerchantOwnership(TypedDict):
    merchant_token: str
    business_id: str
    active_owner_id: str
    has_unexpired_dm: bool
    time_start: str
    time_end: Optional[str]
    is_current: bool

class AMInfo(TypedDict):
    merchant_token: str
    treatment_group: str
    am_team: str
    am_name: str
    sfdc_owner_id: str
    treatment_month: str
    treatment_quarter: str

class BusinessInfo(TypedDict):
    business_id: str
    business_name: str
    merchant_cluster_name: str
    user_token: str
    ultimate_parent_account_id: str
    account_id: str
    is_ultimate_parent: bool

class MerchantSummary(TypedDict):
    merchant_token: Optional[str]
    business_id: Optional[str]
    business_name: Optional[str]
    current_am: Optional[str]
    am_team: Optional[str]
    is_current: Optional[bool]
    last_updated: Optional[str]

class MerchantResponse(TypedDict):
    status: str
    summary: Optional[MerchantSummary]
    details: Optional[Dict[str, Union[MerchantOwnership, AMInfo, BusinessInfo]]]
    data_sources: List[str]
    message: Optional[str]

class TableInfo(TypedDict):
    name: str
    schema: str
    database: str
    kind: str

class TablesResponse(TypedDict):
    status: str
    tables: Optional[List[TableInfo]]
    message: Optional[str]

class ConnectionResponse(TypedDict):
    status: str
    role: Optional[str]
    message: Optional[str]


"""Configuration management for SmartHub extension."""
import os
from dataclasses import dataclass
from typing import Optional

@dataclass
class Config:
    """SmartHub extension configuration."""
    snowflake_user: str = os.getenv("SNOWFLAKE_USER", "")
    snowflake_account: str = os.getenv("SNOWFLAKE_ACCOUNT", "square")
    snowflake_role: str = os.getenv("SNOWFLAKE_ROLE", "ANALYST_MERCH_GROWTH")
    snowflake_warehouse: str = os.getenv("SNOWFLAKE_WAREHOUSE", "ANALYST_WH")
    log_file: str = os.getenv("SMARTHUB_LOG_FILE", "/tmp/smarthub_mcp.log")
    debug: bool = os.getenv("SMARTHUB_DEBUG", "").lower() == "true"

    @property
    def is_valid(self) -> bool:
        """Check if configuration is valid."""
        return bool(self.snowflake_user)

    def validate(self) -> Optional[str]:
        """Validate configuration and return error message if invalid."""
        if not self.snowflake_user:
            return "SNOWFLAKE_USER environment variable is required"
        return None

# Global config instance
config = Config()


"""Snowflake utility functions for SmartHub extension."""
import os
from datetime import datetime
from typing import Any

import snowflake.connector
from snowflake.connector import SnowflakeConnection

from ..config import config

def log_to_file(message: str) -> None:
    """Write a log message to the configured log file.
    
    Args:
        message: The message to log
    """
    timestamp = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    log_message = f"[{timestamp}] {message}\n"
    
    try:
        with open(config.log_file, "a") as f:
            f.write(log_message)
    except Exception as e:
        print(f"Failed to write to log file: {e}")
        print(log_message)

def get_snowflake_connection() -> SnowflakeConnection:
    """Get a connection to Snowflake using configured credentials.
    
    Returns:
        SnowflakeConnection: An authenticated Snowflake connection
    
    Raises:
        ValueError: If required configuration is missing
        Exception: If connection fails
    """
    if error := config.validate():
        raise ValueError(error)
        
    try:
        return snowflake.connector.connect(
            user=config.snowflake_user,
            account=config.snowflake_account,
            authenticator="externalbrowser",
            role=config.snowflake_role,
            warehouse=config.snowflake_warehouse
        )
    except Exception as e:
        log_to_file(f"Failed to connect to Snowflake: {str(e)}")
        raise


from setuptools import setup, find_packages

setup(
    name="mcp-smarthub",
    version="0.2.6",
    packages=find_packages(include=["mcp_smarthub", "mcp_smarthub.*"]),
    install_requires=[
        "mcp>=1.4.1",
        "snowflake-connector-python>=3.14.0",
        "uvicorn>=0.34.0",
        "fastapi>=0.109.0",
        "typing-extensions>=4.9.0",
    ],
    entry_points={
        "console_scripts": ["mcp_smarthub=mcp_smarthub.server:run_server"],
        "mcp.v1": ["mcp_smarthub=mcp_smarthub.server:mcp"],
    },
)


"""SmartHub MCP Server - Main server implementation."""
from typing import Dict, Any

from fastapi import FastAPI, HTTPException
from mcp.server import FastMCP

from .config import config
from .types import (
    ConnectionResponse,
    TablesResponse,
    MerchantResponse,
    TableInfo,
    MerchantSummary
)
from .utils.snowflake_utils import get_snowflake_connection, log_to_file

# Initialize FastAPI app
app = FastAPI(
    title="SmartHub MCP",
    description="SmartHub data access extension for Goose",
    version="0.1.6"
)

# Initialize MCP server
mcp = FastMCP(name="smarthub")

# Define core functionality
async def _test_snowflake_connection() -> ConnectionResponse:
    """Test if we can connect to and query the SmartHub Snowflake tables."""
    log_to_file("\n=== Testing Snowflake Connection ===")
    try:
        conn = get_snowflake_connection()
        cur = conn.cursor()
        
        cur.execute("SELECT current_role()")
        role = cur.fetchone()[0]
        
        return ConnectionResponse(
            status="success",
            role=str(role),
            message=None
        )
        
    except Exception as e:
        log_to_file(f"Connection test failed: {str(e)}")
        return ConnectionResponse(
            status="error",
            role=None,
            message=str(e)
        )

async def _list_available_tables() -> TablesResponse:
    """List all tables available in the APP_MERCH_GROWTH database."""
    log_to_file("Listing available tables...")
    try:
        conn = get_snowflake_connection()
        cur = conn.cursor()
        
        # Get table info
        cur.execute("""
            SHOW TABLES IN DATABASE APP_MERCH_GROWTH;
        """)
        tables = cur.fetchall()
        
        # Format table info
        table_info = [TableInfo(
            name=str(row[1]),
            schema=str(row[2]),
            database=str(row[0]),
            kind=str(row[3])
        ) for row in tables]
        
        return TablesResponse(
            status="success",
            tables=table_info,
            message=None
        )
        
    except Exception as e:
        log_to_file(f"Failed to list tables: {str(e)}")
        return TablesResponse(
            status="error",
            tables=None,
            message=str(e)
        )

async def _get_merchant_info(merchant_token: str) -> MerchantResponse:
    """Get comprehensive information about a merchant using their token or business ID."""
    log_to_file(f"Fetching merchant info for token/id: {merchant_token}")
    try:
        conn = get_snowflake_connection()
        cur = conn.cursor()
        
        # First, let's try to find the merchant in any available tables
        merchant_data: Dict[str, Any] = {}
        data_sources = []
        
        # Convert input to string if it's not already
        merchant_token = str(merchant_token)
        
        # Check if input is numeric (business ID)
        is_business_id = merchant_token.replace("-", "").isdigit()
        log_to_file(f"Input appears to be a {'business ID' if is_business_id else 'merchant token'}")
        
        # First check DIM_AM_OWNERSHIP_HISTORICAL
        if not is_business_id:
            cur.execute("""
                SELECT 
                    MERCHANT_TOKEN,
                    BUSINESS_ID,
                    ACTIVE_OWNER_ID,
                    HAS_UNEXPIRED_DM,
                    TIME_START,
                    TIME_END,
                    IS_CURRENT
                FROM APP_MERCH_GROWTH.PUBLIC.DIM_AM_OWNERSHIP_HISTORICAL
                WHERE MERCHANT_TOKEN = %s
                AND IS_CURRENT = TRUE
                ORDER BY TIME_START DESC
                LIMIT 1
            """, (merchant_token,))
            
            result = cur.fetchone()
            if result:
                column_names = [desc[0].lower() for desc in cur.description]
                ownership_data = dict(zip(column_names, [str(val) if val is not None else None for val in result]))
                merchant_data["ownership"] = ownership_data
                data_sources.append("DIM_AM_OWNERSHIP_HISTORICAL")
                
                # If we found a business_id, store it for later
                if ownership_data.get("business_id"):
                    merchant_token = ownership_data["business_id"]
                    is_business_id = True
        
        # Check ITD_RUN_MERCHANTS_COMBINED_AM_TEAM
        if not is_business_id:
            cur.execute("""
                SELECT 
                    MERCHANT_TOKEN,
                    TREATMENT_GROUP,
                    AM_TEAM,
                    PREFERRED_NAME as AM_NAME,
                    SFDC_OWNER_ID,
                    TREATMENT_MONTH,
                    TREATMENT_QUARTER
                FROM APP_MERCH_GROWTH.PUBLIC.ITD_RUN_MERCHANTS_COMBINED_AM_TEAM
                WHERE MERCHANT_TOKEN = %s
                ORDER BY TREATMENT_MONTH DESC
                LIMIT 1
            """, (merchant_token,))
            
            result = cur.fetchone()
            if result:
                column_names = [desc[0].lower() for desc in cur.description]
                am_data = dict(zip(column_names, [str(val) if val is not None else None for val in result]))
                merchant_data["am_info"] = am_data
                data_sources.append("ITD_RUN_MERCHANTS_COMBINED_AM_TEAM")
        
        # Check PAYOUT_TOOL_BUSINESS_ID_MAP_EXAMPLE
        query = """
            SELECT 
                BUSINESS_ID,
                BUSINESS_NAME,
                MERCHANT_CLUSTER_NAME,
                USER_TOKEN,
                ULTIMATE_PARENT_ACCOUNT_ID,
                ACCOUNT_ID,
                IS_ULTIMATE_PARENT
            FROM APP_MERCH_GROWTH.PUBLIC.PAYOUT_TOOL_BUSINESS_ID_MAP_EXAMPLE
            WHERE {column} = '{value}'
        """.format(
            column="BUSINESS_ID" if is_business_id else "USER_TOKEN",
            value=merchant_token
        )
        
        cur.execute(query)
        result = cur.fetchone()
        if result:
            column_names = [desc[0].lower() for desc in cur.description]
            business_data = dict(zip(column_names, [str(val) if val is not None else None for val in result]))
            merchant_data["business"] = business_data
            data_sources.append("PAYOUT_TOOL_BUSINESS_ID_MAP_EXAMPLE")
            
            # If we found data by business_id, also check for AM info using the user_token
            if is_business_id and business_data.get("user_token") and "am_info" not in merchant_data:
                cur.execute("""
                    SELECT 
                        MERCHANT_TOKEN,
                        TREATMENT_GROUP,
                        AM_TEAM,
                        PREFERRED_NAME as AM_NAME,
                        SFDC_OWNER_ID,
                        TREATMENT_MONTH,
                        TREATMENT_QUARTER
                    FROM APP_MERCH_GROWTH.PUBLIC.ITD_RUN_MERCHANTS_COMBINED_AM_TEAM
                    WHERE MERCHANT_TOKEN = %s
                    ORDER BY TREATMENT_MONTH DESC
                    LIMIT 1
                """, (business_data["user_token"],))
                
                result = cur.fetchone()
                if result:
                    column_names = [desc[0].lower() for desc in cur.description]
                    am_data = dict(zip(column_names, [str(val) if val is not None else None for val in result]))
                    merchant_data["am_info"] = am_data
                    if "ITD_RUN_MERCHANTS_COMBINED_AM_TEAM" not in data_sources:
                        data_sources.append("ITD_RUN_MERCHANTS_COMBINED_AM_TEAM")
        
        if merchant_data:
            # Add summary section
            summary = MerchantSummary(
                merchant_token=merchant_data.get("ownership", {}).get("merchant_token") or 
                              merchant_data.get("am_info", {}).get("merchant_token") or 
                              merchant_data.get("business", {}).get("user_token"),
                business_id=merchant_data.get("ownership", {}).get("business_id") or 
                           merchant_data.get("business", {}).get("business_id"),
                business_name=merchant_data.get("business", {}).get("business_name"),
                current_am=merchant_data.get("am_info", {}).get("am_name"),
                am_team=merchant_data.get("am_info", {}).get("am_team"),
                is_current=merchant_data.get("ownership", {}).get("is_current"),
                last_updated=merchant_data.get("am_info", {}).get("treatment_month")
            )
            
            return MerchantResponse(
                status="success",
                summary=summary,
                details=merchant_data,
                data_sources=data_sources,
                message=None
            )
        
        return MerchantResponse(
            status="error",
            summary=None,
            details=None,
            data_sources=[],
            message=f"No data found for {'business ID' if is_business_id else 'merchant token'}: {merchant_token}"
        )
        
    except Exception as e:
        log_to_file(f"Failed to get merchant info: {str(e)}")
        return MerchantResponse(
            status="error",
            summary=None,
            details=None,
            data_sources=[],
            message=str(e)
        )

# Register MCP tools
@mcp.tool()
async def test_snowflake_connection() -> ConnectionResponse:
    """Test if we can connect to and query the SmartHub Snowflake tables."""
    return await _test_snowflake_connection()

@mcp.tool()
async def list_available_tables() -> TablesResponse:
    """List all tables available in the APP_MERCH_GROWTH database."""
    return await _list_available_tables()

@mcp.tool()
async def get_merchant_info(merchant_token: str) -> MerchantResponse:
    """Get comprehensive information about a merchant using their token or business ID."""
    return await _get_merchant_info(merchant_token)

@app.on_event("startup")
async def startup_event() -> None:
    """Validate configuration on startup."""
    if error := config.validate():
        raise HTTPException(status_code=500, detail=error)
    log_to_file("SmartHub MCP Server starting up...")

def run_server() -> None:
    """Run the MCP server."""
    mcp.run()


"""CLI entry point for SmartHub MCP extension"""
import typer
import uvicorn
from fastapi import FastAPI
from mcp.server import FastMCP

from smarthub_extension.server import mcp

app = FastAPI()
cli = typer.Typer()

@cli.command()
def run(
    host: str = "127.0.0.1",
    port: int = 8000,
    transport: str = "stdio"
):
    """Run the SmartHub MCP extension server"""
    if transport == "stdio":
        # Run in stdio mode (for Goose)
        mcp.run(transport="stdio")
    else:
        # Run as FastAPI server
        app.mount("/mcp", mcp.app)
        uvicorn.run(app, host=host, port=port)